% function to compute initilization using K-Means
%
% inputs:
% feats_a: input features (cell)
% nStates_a: number of states
% nMixes_a: number of mixtures per state
% covType_a: cov. type
% structure_a: stucture ergodic or left_to_right
%
% output:
% model_a: output HMM  initilized
%
function model_a = calculate_init_params(feats_a,nStates_a,nMixs_a,covType_a,structure_a)

% initilize transmat,prior,mix weights
model_a.prior = normalise(rand(nStates_a,1));

% 
if strcmp(structure_a,'ergodic')
    model_a.transmat = mk_stochastic(rand(nStates_a,nStates_a));
elseif strcmp(structure_a,'left_to_right')
    model_a.transmat = mk_leftright_transmat(nStates_a,0.6);
end

model_a.mixmat = mk_stochastic(rand(nStates_a,nMixs_a));

feats=[];
for i=1:length(feats_a)
    feats=[feats feats_a{i}];
end
%feats=feats';

D=size(feats,1);
[mu00, cov00] = mixgauss_init(nStates_a*nMixs_a,feats, covType_a);
if isempty(find(isnan(cov00)))~=1
    [tmp00, cov00] = mixgauss_init(nStates_a*nMixs_a,feats, covType_a,'rnd');
end
save('debug');
model_a.mu = reshape(mu00, [D nStates_a nMixs_a]);
model_a.Sigma = reshape(cov00, [D D nStates_a nMixs_a]);


