clear
clc
%%
% add to path
addpath('voicebox'); % some functions for feature extraction etc
addpath(genpath('HMMall')); % add HMM toolbox to the path

% where wavs are?
input_dir='VOLTASpeechSamples';

% feature configuration
%  This is used to control how we extract features
%
W = 'Mtp0dD'; % used to force window being Hamming, triangular, power domain,
              % include c0 (E can be used for log energy but cause error
              % for your files since you have artificially add zeros to
              % files;if you want to use E  then remove the artificial zeros)
              % and include first and second derivitives
nc = 12; % number of cepstral features exluding 0's

frame_len = 25; % in msec
frame_inc = 10; % frame increament in msec              
fl = 1; % lowe cutoff  for filter bank in Hz
fh = 8000; % higher cuttoff for filter bank in Hz


% HMM  paramters
% config the HMM model here
%
nStates = 5; %number of states for each HMM 
nMixs = 10; %number of mixture component per state (e.g. 10 Gaussians)
covType='diag'; % covariance type (diag,full)
hmmStructure='ergodic'; % ergodic or  left_to_right
max_itr =5;  % max iterations 


% read list of files (Class1)
fid = fopen('Class1.txt');
files1 = textscan(fid, '%s','delimiter', '\n');
fclose(fid);
% class2
fid = fopen('Class2.txt');
files2 = textscan(fid, '%s','delimiter', '\n');
fclose(fid);

% feature extraction  for class1
for i=1:length(files1{1})
    % read raw signal
    [x1,fs1] = readwav(strcat(input_dir,'/',files1{1}{i}));
    fs1=8000    
    frame_len_sample = frame_len * fs1*0.001;
    frame_inc_sample = frame_inc * fs1*0.001;
    p = floor(3*log(fs1));% number of filters (you can modify this)

    % compute the feature
    % feat is a matrix with N(number of frames) colums and M ( number of features)
    % rows
    %
    
    feats1{i} = melcepst(x1,fs1,W,nc,p,frame_len_sample,frame_inc_sample,fl/fs1,fh/fs1)';    

end


% feature extraction  for class1
for i=1:length(files2{1})
    % read raw signal
    [x1,fs1] = readwav(strcat(input_dir,'/',files2{1}{i}));
    fs1=8000 
    frame_len_sample = frame_len * fs1*0.001;
    frame_inc_sample = frame_inc * fs1*0.001;
    p = floor(3*log(fs1));% number of filters (you can modify this)

    % compute the feature
    % feat is a matrix with N(number of frames) colums and M ( number of features)
    % rows
    %
    feats2{i} = melcepst(x1,fs1,W,nc,p,frame_len_sample,frame_inc_sample,fl/fs1,fh/fs1)';    

end

% init HMMs
% model structure is ergodic or left_to_right (here I select ergodic)
%

% class1
model1_0 = calculate_init_params(feats1,nStates,nMixs,covType,hmmStructure);

% class2
model2_0 = calculate_init_params(feats2,nStates,nMixs,covType,hmmStructure);


% train HMMs
%

%class 1
[model1_1,LL1] = trainHMM(feats1,model1_0,max_itr);

%class 2
[model2_1,LL2] = trainHMM(feats2,model2_0,max_itr);

% decode

[decoded1,LL1]=decodeHMM(feats1,model1_1,model2_1,[0;0]);
[decoded2,LL2]=decodeHMM(feats2,model1_1,model2_1,[0;0]);


% eval
ind1=find(decoded1~=1); % find all decoded examples in list 1 that are not decoded as class 1
ind2=find(decoded2~=2); % find all decoded examples in list 2 that are not decoded as class 2

% class1 --> class2 errors
fprintf('Error type 1:%f\n',100.0*length(ind1)/length(decoded1));

% class2 --> class1 errors
fprintf('Error type 2:%f\n',100.0*length(ind2)/length(decoded2));

% total errors
fprintf('Total Error:%f\n',100.0*(length(ind1)+length(ind2))/(length(decoded1)+length(decoded2)));



